/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

//////////////
// murasaki project
// murasaki_mpi.h
// provides MPI-specific functions
//////////////

#ifdef MURASAKI_MPI
#ifndef MURASAKI_MPI_H__
#define MURASAKI_MPI_H__ 1

#include "sequence.h"
#include "dinkymath.h"
#include <mpi.h>
#include <vector>
#include <list>

using namespace std; //oof. once you start, you can't stop.

//signals
#define MURASAKI_MPI_REQ_SIZE 0
#define MURASAKI_MPI_REQ_LKUP_NOMATCH 1
#define MURASAKI_MPI_REQ_LKUP_MATCH 2
#define MURASAKI_MPI_REQ_QUIT  999

//types
enum MPI_Job {JOB_HASHER=0,JOB_STORAGE,JOB_MAX};

struct mpi_req{
  int type;
  HashKey key;
  HashVal val;
};

class MPI_HashMessage {
 public:
  word key;
  SeqIdx seqno;
  SeqPos pos;
  MPI_HashMessage(const word& key,const Location& l);
  void set(const word& key,const Location& l);
  MPI_HashMessage();
 private:
};

//have to tell MPI what these things are
extern MPI_Datatype MPI_HashMessage_type;

const int mpi_bufferSize=50000; //for some reason large values cause large problems on mx (stops receiving after the first packet)

class MessageBlock {
 public:
  int used;
  int stored;
  MPI_HashMessage messages[mpi_bufferSize];
  MPI_Request request;
  MessageBlock();
  MessageBlock(const MessageBlock &a); //fake-copy
};

class MPI_AnchorHeader {
 public:
  word hashKey;
  word count;
  MPI_AnchorHeader(word key);
  MPI_AnchorHeader();
};

class AnchorBlock {
 public:
  vector<Location> locs;
  MPI_AnchorHeader header;
  MPI_Request headerReq,bodyReq;
  int headerStored,bodyStored;
  int owner;

  AnchorBlock();
  AnchorBlock(word hashKey);
  AnchorBlock(const AnchorBlock &a); //fake-copy
};

//globals

extern int mpi_id;
extern int mpi_procs;
extern bool mpi_capable;
extern word mpi_totalMemory,mpi_totalStorage;
extern vector<double> mpi_storeShare;
extern vector<pair<word,word> > mpi_storeOwn;
extern map<word,int> mpi_storeBrk; // start of hash region -> mpi rank
extern map<word,int> mpi_storeBrkStop; // stop+1 of hash region -> id
extern word mpi_myStoreOffset;
extern int mpi_finalAssembler;

extern vector<string> mpi_hostnames;
extern map<string,int> mpi_hostLeader_byName;
extern vector<int> mpi_hostLeader;//lowest id on same host
extern bool mpi_isHostLeader,mpi_usingShm;

extern vector<long> mpi_hashCount,mpi_storeCount,mpi_extractCount,mpi_mergeCount,mpi_extractLocCount;
extern vector<double> mpi_workTime;

extern vector<double> mpi_hashShare;
extern vector<pair<word,word> > mpi_hashOwn;
extern map<word,int> mpi_hashBrk; //start of sequence -> mpi rank
extern map<word,Location> mpi_hashPoints; //maps each hashBrk to an actual Location

extern word mpi_total_hash_size;
extern vector<word> mpi_worldMemory;
extern vector<MPI_Job> mpi_jobs;
extern vector<int> mpi_jobCount;
extern const char *MPI_jobNames[];

extern int mpi_sysv_projid;

extern MPI_Comm mpi_leaders_comm,mpi_localhost_comm;
extern int mpi_myHostId,mpi_myLeaderRank,mpi_myLocalRank;

//functions
void mpi_init();
void mpi_initJobs();
void mpi_types_init();
void mpi_fillHashPoints();
void mpi_syncSeqCounts();

void mpi_hasher_client_mode(BitSequence *pat);
void mpi_storage_client_mode();
void mpi_storage_store(list<MessageBlock> &toStore);

void mpi_merge_client_mode();
void mpi_extract_client_mode(BitSequence *pat);
void mpi_extract_checkSentQueue(list<AnchorBlock*> &sent);
void mpi_procAnchorBlock(list<AnchorBlock*> &todo);

void mpi_hashAndStore_client_mode(BitSequence *pat);
void mpi_hashSeqLocal(BitSequence *bitseq,BitSequence *pat,int sign,Sequence *s);

void mpi_write_histogram();
#endif
#endif
