/*
Murasaki - multiple genome global alignment program
Copyright (C) 2006-2007 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

////////////////
// timing.h
// defs for timing tools
////////////////

#ifndef __TIMING_H
#define __TIMING_H

#include <sys/time.h>
#include <time.h>
#include <iostream>

#define TIME_DOUBLE(a) ((double)(a.tv_sec)+(double)(a.tv_usec)/1000000.0)

extern bool timing_useHumanTime;

using namespace std;

class Timer {
 public:
  struct timeval tv;
  Timer();
  Timer(double t);
  void reset(); //reset timer to now
  double asDouble() const;
  
  friend Timer operator- (const Timer &a,const Timer &b);
  friend double diff(const Timer &a,const Timer &b);
  friend Timer operator*(const Timer &a,double factor);
  friend Timer operator*(double factor,const Timer &a);
  friend Timer operator/(const Timer &a,double factor);
  friend Timer operator/(double factor,const Timer &a);
  friend ostream& operator <<(ostream &os,const Timer &obj);
};

string elapsed(Timer a,Timer b);
double diff(const Timer &a,const Timer &b);
string humanTime(double dur);

#endif
